/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "animated_pixmap.hpp"
#include <algorithm>

namespace duck {
AnimatedPixmap::AnimatedPixmap (bool loop) :
	m_frame(0),
	m_loop(loop)
{
}

AnimatedPixmap::~AnimatedPixmap() noexcept = default;

void AnimatedPixmap::push_back (QPixmap&& frame) {
	m_frames.push_back(std::move(frame));
}

std::size_t AnimatedPixmap::frame_count() const {
	return m_frames.size();
}

QSize AnimatedPixmap::size() const {
	return {
		(m_frames.empty() ? EmptyPixmapWidth : m_frames.front().width()),
		(m_frames.empty() ? EmptyPixmapHeight : m_frames.front().height())
	};
}

void AnimatedPixmap::advance_frame() {
	++m_frame;
	if (m_loop)
		m_frame = (not m_frames.empty() ? m_frame % m_frames.size() : 0);
	else
		m_frame = std::min(m_frame, m_frames.size());
}

const QPixmap& AnimatedPixmap::current_frame() const {
	return m_frames[m_frame];
}
} //namespace duck
