/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QPixmap>
#include <vector>

namespace duck {
class AnimatedPixmap {
public:
	static const constexpr int EmptyPixmapWidth = -1;
	static const constexpr int EmptyPixmapHeight = EmptyPixmapWidth;

	AnimatedPixmap (AnimatedPixmap&&) = default;
	AnimatedPixmap (const AnimatedPixmap&) = delete;
	AnimatedPixmap (bool loop);
	~AnimatedPixmap() noexcept;

	AnimatedPixmap& operator= (AnimatedPixmap&&) = default;

	void push_back (QPixmap&& frame);
	std::size_t frame_count() const;
	QSize size() const;

	void advance_frame();
	const QPixmap& current_frame() const;

private:
	std::vector<QPixmap> m_frames;
	std::size_t m_frame;
	bool m_loop;
};
} //namespace duck
